/*
 * This file is part of din.
 *
 * din is copyright (c) 2006 - 2012 S Jagannathan <jag@dinisnoise.org>
 * For more information, please visit http://dinisnoise.org
 *
 * din is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * din is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with din.  If not, see <http://www.gnu.org/licenses/>.
 *
*/

#ifndef __widget
#define __widget

#include "box.h"
#include "color.h"

#include <vector>

struct ui;

class widget { // ui widget

	box<int> extents;

	color clr; // color

	int hover; // mouse over widget?
	
	int posx, posy; // position in screen space
	
	std::vector<widget *> children;
	typedef std::vector<widget *>::size_type size_v;
	
	int moveable; // is moveable?
	
	int visible; // is visible?
	
	std::vector <ui *> screens; // screens where shown
		
	public:

		static widget* focus;

		widget (int l = 0, int b = 0, int r = 0, int t = 0);

		const box<int>& get_extents () const {return extents;}
		void set_extents (int l, int b, int r, int t);
		void set_extents (const box<int>& ext) { extents = ext;}

		void set_color (float r, float g, float b) { clr.r = r; clr.g = g; clr.b = b; }
		void set_color (const color& c) {clr.r = c.r; clr.g = c.g; clr.b = c.b; }
		const color& get_color () const {return clr;}
		
		void set_pos (int x, int y);
		void get_pos (int& x, int& y);

		int hovering () {return hover;}
		
		virtual int handle_input ();	
		virtual void draw ();

		void draw_bbox ();
		
		void add_child (widget *w);
		
		int is_moveable () const {return moveable;}
		void set_moveable (int m) {moveable = m;}
		void move (int dx, int dy);
		
		void add_screen (ui* scr);
		int is_screen (ui* scr);
		const std::vector<ui*>& get_screens () const {return screens;}

		int is_visible () const {return visible;}
		void show ();
		void hide ();
		
		int enable_;
		void enable (int e) {enable_ = e;} 
		int enable () const {return enable_;}
		
};

// listeners for widgets
//
template <typename W> struct change_listener {
	virtual void changed (const W& w) = 0;
};

#endif
