/*
 * This file is part of din.
 *
 * din is copyright (c) 2006 - 2012 S Jagannathan <jag@dinisnoise.org>
 * For more information, please visit http://dinisnoise.org
 *
 * din is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * din is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with din.  If not, see <http://www.gnu.org/licenses/>.
 *
*/
#include "ui_list.h"
#include "din.h"
#include "morse_code.h"
#include "console.h"
#include "curve_editor.h"
#include "tcl_interp.h"
#include "command.h"
#include "chrono.h"
#include "font_editor.h"
#include "key_consts.h"
#include "keyboard_keyboard.h"
#include "fft.h"

using namespace std;

extern console cons;
extern cmdlist cmdlst;
extern din din0;
extern morse_code mc;
extern curve_editor delayed;
extern string dotdin;
extern tcl_interp* p_interp;
extern font fnt;
extern curve_editor waved, comed, octed, veled, dmod_ed;

extern din din0;

extern keyboard_keyboard keybd2;
extern fft fft0;

extern ui_list uis;

extern audio_out aout;

extern int quit_now;

ui_list::ui_list () :  
fed (fnt, "font.ed") {

  current = 0;
  prev = 0;

  crved = 0;
  
	ui* scrs [] = {
		&din0,
		&din0.waved,
		&keybd2.waved,
		&din0.moded,
		&din0.gated,
		&delayed,
		&octed,
		&din0.droneed,
		&comed,
		&mc.ed,
		&dmod_ed,
		&fed,
		&keybd2.attacked,
		&keybd2.decayed,
		&keybd2.veled,
		0
	};
	
	for (ui** p = scrs; *p != 0; ++p) {
		uis.push_back (*p);
	}

  esct = -1;
  
  setup_widgets ();

}

void ui_list::set_current (ui* u, int ised) {

  if (current) {
    current->leave ();
    prev = current;
  }

  current = u;
  current->enter ();

  if (ised) {
    crved = dynamic_cast<curve_editor*>(u);
    if (crved->sine_enabled) fft0.go (crved->curveinfo[0].curve);
  } else {
    crved = 0;
    uis[0] = current;
  }

  cons << console::yellow << "@ " << current->name << eol;

}

bool ui_list::handle_input () {

  current->calc_win_mouse ();
  
  din0.bg (); // bg for handling phrasor on microtonal-keyboard
  for (vector<ui*>::size_type i = 1, j = uis.size(); i < j; ++i) uis[i]->bg (); // bg for editors
  
  eval_fade (fdr_voice, w_voice);
	flash_gater (); // flashes the gater checkbox in sync with gater value
  eval_fade (fdr_delay, w_delay);
    
	if (widget::focus == 0) {
		for (vector<widget*>::size_type i = 0, j = widgets.size (); i < j; ++i) {
			widget* wi = widgets[i];
			if (wi->is_screen (current) && wi->enable()) {
				if (wi->handle_input ()) return true;
			}
		}
	} else {
		widget* wf = widget::focus;
		if (wf->handle_input ()) return true;
	}

  if (cons.command_mode == 0) {
		
    current->handle_input ();

    if (keypressed (k_enter)) { // calc tap bpm
      extern nano_chrono ui_clk;
      extern double TAP_BPM;
      static double last_tapt = ui_clk.secs;
      double dt = ui_clk.secs - last_tapt;
      if (dt) TAP_BPM = 60.0 / dt;
      extern tcl_interp interpreter;
      if (keydown (k_lctrl)) cons ("set resetbeat 1");
      Tcl_UpdateLinkedVar (interpreter.interp, "tapbpm");
      last_tapt = ui_clk.secs;
    }

    extern string INSTRUMENT;

    if (keypressed (k_1)) { // switch instrument
      if (crved == 0) { 
        if (current == &din0) {
          set_current (&keybd2, 0);
          cons ("setup-editors keyboard_keyboard");
          INSTRUMENT = "keyboard_keyboard";
        } else {
          set_current (&din0, 0);
          cons ("setup-editors microtonal_keyboard");
          INSTRUMENT = "microtonal_keyboard";
        }
      } else set_current (uis[0], 0);
    } else {

			for (int i = 1; i < MAX_EDITORS; ++i) {
				if (keypressed (key[i])) {
					if (ed[i]) set_current (ed[i]);
					break;
				}
			}
		}

  }

  extern nano_chrono ui_clk;
  if (keypressed (k_esc)) {
    if (cons.command_mode || (current != uis[0])) {
      set_current (uis[0], 0);
      cons.command_mode = 0;
      esct = -1;
    } else {
      esc:
      if (esct == -1) {
        cons << console::red << "Press ESC again to quit" << eol;
        esct = ui_clk.secs;
      } else {
        static const float ESC_TIMEOUT = 1; // in seconds
        float dt = ui_clk.secs - esct;
        if (dt > ESC_TIMEOUT) {
          esct = -1;
          goto esc;
        } else {
					din0.save_scale ();
					din0.dinfo.save ();
					din0.delete_all_drones ();
					w_voice.turn_off ();
					w_delay.turn_off ();
					quit_now = 1;
					cons << console::green << "Goodbye!" << eol;
        }
      }
    }
  }

  return true;
}

bool ui_list::set_editor (const string& name, int screen) {

  if (screen > 0) {

    ui* ei = 0;
    for (int i = 0, j = uis.size(); i < j; ++i) {
      if (uis[i]->name == name) {
        ei = uis[i];
        break;
      }
    }
    if (ei) {
      ed [screen - 1] = ei;
      return true;
    } else {
      cmdlst.result = "bad editor name";
      return false;
    }
  } else {
    cmdlst.result = "bad screen number (valid: 1 to 8)";
    return true;
  }
}

void ui_list::draw () {
	
  glClear (GL_COLOR_BUFFER_BIT);
  
  current->draw ();
  
  glMatrixMode (GL_PROJECTION);
  glPushMatrix ();
  glLoadIdentity ();
  glOrtho (0, view.xmax, 0, view.ymax, -1, 1);
  
  glMatrixMode (GL_MODELVIEW);
  glPushMatrix ();
  glLoadIdentity ();
  
  for (vector<widget*>::size_type i = 0, j = widgets.size (); i < j; ++i) {
		widget* wi = widgets[i];
		if (wi->is_screen (current)) wi->draw ();
	}
	
	glPopMatrix ();
	glMatrixMode (GL_PROJECTION);
	glPopMatrix ();
  glMatrixMode (GL_MODELVIEW);
	
}

void ui_list::setup_widgets () {
	
	/*w_rate.add_screen (&din0);
	w_rate.set_pos (10, 90);
	w_rate.set_color (0.0, 0.45, 1);
	w_rate.set_values (0, 480, 0);

	w_depth.set_values (0, 100, 0);
	w_depth.add_screen (&din0);
	w_depth.set_pos (10, 70);
	w_depth.set_color (1, 0.45, 0.9);*/

	w_voice.set_label ("Voice");
	w_voice.add_screen (&din0);
	w_voice.set_listener (&vlis);
	w_voice.colorize (0);
	
	w_gater.set_label ("Gater");
	w_gater.add_screen (&din0);
	w_gater.set_listener (&glis);
	w_gater.colorize (0);
	
	w_delay.set_label ("Delay");
	w_delay.add_screen (&din0);
	w_delay.add_screen (&keybd2);
	w_delay.set_listener (&dlis);
	w_delay.colorize (0);
	
	w_compress.set_label ("Compressor");
	w_compress.add_screen (&din0);
	w_compress.add_screen (&keybd2);
	w_compress.set_listener (&clis);
	
	w_ticker.donate.set_label ("Donate");
	w_ticker.add_screen (&din0);
	w_ticker.add_screen (&keybd2);
	
	cons.add_screen (0); // 0 for all
	
	widgets.push_back (&w_voice);
	widgets.push_back (&w_gater);
	widgets.push_back (&w_delay);
	widgets.push_back (&w_compress);
	widgets.push_back (&w_ticker);
	widgets.push_back (&cons);
	
	w_voice.set_state (din0.dinfo.voice);
	w_gater.set_state (din0.dinfo.gater);
	w_delay.set_state (din0.dinfo.delay);
	w_compress.set_state (din0.dinfo.compress);
				
}

void ui_list::update_widgets () {

	button* wd [] = {&w_voice, &w_gater, &w_delay, &w_compress, &w_ticker.donate};
	int n = 5, dx = 50, w = 0;
	for (int i = 0; i < n; ++i) w += wd[i]->get_extents ().width;
	w += n * dx;
	
	int lh = get_line_height ();
	int x = (view.xmax - w) / 2, y = lh + 2;
	for (int i = 0; i < n; ++i) {
		button& b = *wd[i];		
		const box<int>& e = b.get_extents ();
		b.set_extents (x, y, x + e.width, y + e.height);
		b.set_pos (x, y);
		x = e.right + dx;
	}
	
	w_ticker.toggle.set_label (">>>");
	w_ticker.toggle.set_pos (0, 2);
	const box<int>& e = w_ticker.toggle.get_extents ();
	w_ticker.xstart = e.right;
	w_ticker.dy = 1;
	w_ticker.twin (w_ticker.xstart, e.bottom - w_ticker.dy, view.xmax, e.bottom + lh);
	w_ticker.startx = view.xmax;	
	int msgw = get_char_width (ticker_tape::message);
	w_ticker.leftxt = -msgw; 
	w_ticker.rightxt = view.xmax;
	

}

void ui_list::flash_gater () {
	static color clr [2] = {checkbutton::off_color, checkbutton::on_color};
	float dg = aout.gatr [aout.samples_per_channel - 1];
	if (fdr_gater.eval ()) {
		w_gater.enable (0);
		w_gater.blend_on_off_color (fdr_gater.alpha);
		color& c = const_cast<color&> (w_gater.get_color ());
		c *= dg;
	} else {
		w_gater.enable (1);
		int gi = din0.dinfo.gater;
		w_gater.set_color (dg * clr [gi].r, dg * clr [gi].g, dg * clr[gi].b);
	}
}

float ui_list::eval_fade (fader& fdr, checkbutton& cb) {
  if (fdr.eval ()) {
		cb.enable (0);
		cb.blend_on_off_color (fdr.alpha); 
	} else cb.enable (1);
	
	return fdr.amount; 	
} 

void setup_fade (fader& fdr, int& target, int what) {
  if (fdr.on == 0) {
		target = what;
		if (what) fdr.set (0, 1); else fdr.set (1, 0);
	}
}

void voice__listener::changed (const checkbutton& cb) {
	int what = cb.is_on ();
	setup_fade (uis.fdr_voice, din0.dinfo.voice, what);
}

void gater__listener::changed (const checkbutton& cb) {
  int what = cb.is_on ();
  setup_fade (uis.fdr_gater, din0.dinfo.gater, what);
}

void delay__listener::changed (const checkbutton& cb) {
  int what = cb.is_on ();
  setup_fade (uis.fdr_delay, din0.dinfo.delay, what);
}

void compress__listener::changed (const checkbutton& cb) {
  din0.dinfo.compress = cb.is_on ();
}
